/*!
 * type-reverse <https://github.com/whizkydee/type-reverse>
 *
 * Copyright (c) 2017-present, Olaolu Olawuyi.
 * Released under the MIT License.
 */

'use strict';

/**
 * 🦄 Lightweight reverse utility around strings, arrays, numbers and more.
 *
 * @name reverse
 * @alias inverse
 * @param {String|Number|Array|NodeList|Boolean} `input`
 * @param {?Object} `options`
 * @return {undefined}
 * @api public
 */

Object.defineProperty(exports, "__esModule", {
  value: !0
});

var _is = require('./util/is');

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) arr2[i] = arr[i]; return arr2; } return Array.from(arr); }

function reverse(input) {
  var options = 1 < arguments.length && arguments[1] !== undefined ? arguments[1] : {};

  if (input && !(0, _is.supportedTypes)(input)) throw new TypeError('Failed to apply \'reverse\': ' + (0, _is.typeOf)(input) + 's are not supported');

  var then = options.then;

  options.invert = options.invert || 'index', options.then = then || function (_, v) {
    return v;
  };


  // create a new array, copy the items of the initial
  // into the new then reverse the new array.
  var globArr = _is.is.string(input) || _is.is.array(input) ? [].concat(_toConsumableArray(input)).reverse() : undefined,
      re = /^-/,
      result = void 0;

  switch ((0, _is.typeOf)(input)) {
    case 'string':
      switch (options.invert) {
        case 'index':
          result = globArr.join('');

          break;
        case 'word':
          result = input.split(' ').reverse().join(' ');

      }
      break;

    case 'number':
      // convert the number to string then replace the minus(-) symbol with nothing
      var nStr = String(input).replace(re, '');

      if (/e/.test(nStr)) throw new TypeError('Oops. That number is too large. See https://github.com/whizkydee/type-reverse/blob/master/readme.md#limits for more info.');

      switch (options.invert) {
        case 'sign':
          result = re.test(input) ? Number(+nStr) : Number(-nStr);

          break;
        case 'index':
          result = re.test(input) ? reverse(nStr, { then: function (_, x) {
              return Number(-x);
            } }) : reverse(nStr, { then: function (_, x) {
              return Number(x);
            } });

      }
      break;

    case 'array':
    case 'nodelist':
      result = globArr;

      break;

    case 'boolean':
      result = !input;

      break;

    default:
      result = reverse(input);

  }

  if ('function' === typeof then) return then.call(this, input, result);
  if (then && 'function' !== typeof then) throw new TypeError('Failed to apply \'reverse\': Expected function as second argument, got ' + (0, _is.typeOf)(then) + '.');

  return result;
}

exports.default = reverse;
module.exports = reverse;